require 'oop/non_value_arg'
require 'oop/value'

module OOP
  class BasicRules
    def self.instance
      @instance ||= new
    end

    def self.cast_to_value(arg)
      self.instance.cast_to_value(arg)
    end

    def cast_to_value(arg)
      if arg.respond_to?(:super_mega_private_oop_method_value?) && arg.super_mega_private_oop_method_value?
        return arg
      end

      case arg
      when Value, TrueClass, FalseClass, NilClass, Proc then
        arg
      when Hash then
        Hash[arg.map{|k,v| [cast_to_value(k), cast_to_value(v)] }.to_h].freeze
      when Array then
        arg.map{|v| cast_to_value(v) }.freeze
      when Symbol, Fixnum, String, Float, Time then
        if arg.frozen?
          arg
        else
          arg.freeze
        end
      else
        raise NonValueArg.new(arg.class)
      end
    end
  end
end
