require 'oop/basic_rules'

module OOP
  module ValueObject
    module ClassMethods
      def message(meth, &block)
        define_method(meth) do |*args, &blk|
          args = args.map do |arg|
            BasicRules.cast_to_value(arg)
          end
          BasicRules.cast_to_value(self.instance_exec(*(args + [blk]), &block))
        end
      end

      def super_mega_private_oop_method_value?
        true
      end

      def import(imports)
        imports.each do |name, import|
          define_method(name.to_sym) do
            BasicRules.cast_to_value(import)
          end
        end
      end

      def constructor(*parts, &block)
        define_method(:initialize) do |opts={}|
          opts ||= {}
          if block
            block.call(opts)
          end
          extra_args = (opts.keys - parts)
          missing_args = parts.select do |k|
            !opts.has_key?(k) && !opts.has_key?(k)
          end
          if extra_args.any?
            raise "extra args: #{extra_args}"
          elsif missing_args.any?
            raise "missing args: #{missing_args}"
          end
          @opts = opts;
          opts.each do |k, v|
            k = k.to_sym
            @opts[BasicRules.cast_to_value(k)] = BasicRules.cast_to_value(v)
          end
        end
        parts.each do |p|
          define_method(p.to_sym) do
            @opts[p]
          end
        end
      end
    end

    def super_mega_private_oop_method_value?
      true
    end

    def self.included(klass)
      klass.extend(ClassMethods)
    end
  end
end
